@@ -43,12 +43,12 @@ import ipaddress
 import six
 
 ZONE_PROPERTY_EXPLANATION = """\
-If not specified, you may be prompted to select a zone. `gcloud` will attempt
-to identify the zone by searching for resources in your project. If the zone
-cannot be determined, you will then be prompted with all Google Cloud
-Platform zones.
+If not specified, the user may be prompted to select a zone. `gcloud` will
+attempt to identify the zone by searching for resources in the user's project.
+If the zone cannot be determined, the user will then be prompted with all Google
+Cloud Platform zones.
 
-To avoid prompting when this flag is omitted, you can set the
+To avoid prompting when this flag is omitted, the user can set the
 ``compute/zone'' property:
 
   $ gcloud config set compute/zone ZONE
@@ -270,11 +270,13 @@ def AddImageArgs(parser, enable_snapshots=False):
   image_group.add_argument(
       '--image-family',
       help="""\
-      The family of the image that the boot disk will be initialized
-      with. When a family is specified instead of an image, the latest
-      non-deprecated image associated with that family is used. It is best
-      practice to use `--image-family` when the latest version of an image is
-      needed.
+      The image family for the operating system that the boot disk will
+      be initialized with. Compute Engine offers multiple Linux
+      distributions, some of which are available as both regular and
+      Shielded VM images.  When a family is specified instead of an image,
+      the latest non-deprecated image associated with that family is
+      used. It is best practice to use `--image-family` when the latest
+      version of an image is needed.
 
       By default, ``{default_image_family}'' is assumed for this flag.
       """.format(default_image_family=constants.DEFAULT_IMAGE_FAMILY))
@@ -286,7 +288,7 @@ def AddImageArgs(parser, enable_snapshots=False):
         will be created from. You can provide this as a full URL
         to the snapshot or just the snapshot name. For example, the following
         are valid values:
-          * https://www.googleapis.com/compute/v1/projects/myproject/global/snapshots/snapshot
+          * https://compute.googleapis.com/compute/v1/projects/myproject/global/snapshots/snapshot
           * snapshot
         """)
 
@@ -420,7 +422,7 @@ def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False,
       unit of ``KB'' for kilobyte, ``MB'' for megabyte, ``GB'' for gigabyte,
       or ``TB'' for terabyte. For example, ``10GB'' will produce a 10 gigabyte
       disk. The minimum size a boot disk can have is 10 GB. Disk size must be a
-      multiple of 1 GB. Limit your boot disk size to 2TB to account for MBR
+      multiple of 1 GB. Limit boot disk size to 2TB to account for MBR
       partition table limitations.
       """)
 
@@ -496,7 +498,8 @@ def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False,
 
 def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
                       container_mount_enabled=False, resource_policy=False,
-                      csek_key_file=False):
+                      source_snapshot_csek=False,
+                      image_csek=False):
   """Adds create-disk argument for instances and instance-templates."""
 
   disk_device_name_help = _GetDiskDeviceNameHelp(
@@ -527,19 +530,23 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
       a specific version of an image is needed. If both image and image-family
       flags are omitted a blank disk will be created.
 
-      *image-family*::: The family of the image that the disk will be
-      initialized with. When a family is specified instead of an image,
+      *image-family*::: The image family for the operating system that the boot
+      disk will be initialized with. Compute Engine offers multiple Linux
+      distributions, some of which are available as both regular and
+      Shielded VM images.  When a family is specified instead of an image,
       the latest non-deprecated image associated with that family is
-      used. It is best practice to use image-family when the latest version
-      of an image is needed.
+      used. It is best practice to use --image-family when the latest
+      version of an image is needed.
 
-      *image-project*::: The project that the image or image family
-      belongs to. It is best practice to define image-project.
+      *image-project*::: The Google Cloud project against which all image and
+      image family references will be resolved. It is best practice to define
+      image-project. A full list of available projects can be generated by
+      running `gcloud projects list`.
           * If specifying one of our public images, image-project must be
             provided.
           * If there are several of the same image-family value in multiple
-            projects, image-project must be specified to clarify the image to
-            be used.
+            projects, image-project must be specified to clarify the image to be
+            used.
           * If not specified and either image or image-family is provided, the
             current default project is used.
 
@@ -618,7 +625,7 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
       create the disk. You can provide this as a full URL
       to the snapshot or just the snapshot name. For example, the following
       are valid values:
-        * https://www.googleapis.com/compute/v1/projects/myproject/global/snapshots/snapshot
+        * https://compute.googleapis.com/compute/v1/projects/myproject/global/snapshots/snapshot
         * snapshot
       """
     spec['source-snapshot'] = str
@@ -626,30 +633,49 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
   if resource_policy:
     disk_help += """
       *disk-resource-policy*::: Resource policy that will be applied to created
-      disk. You can provide full or partial URL. See
+      disk. You can provide full or partial URL. For more details see
         * https://cloud.google.com/sdk/gcloud/reference/beta/compute/resource-policies/
         * https://cloud.google.com/compute/docs/disks/scheduled-snapshots
-      for more details about resource-policies.
       """
     spec['disk-resource-policy'] = arg_parsers.ArgList(max_length=1)
 
+  if source_snapshot_csek:
+    disk_help += """
+      *source-snapshot-csek-required*::: The CSK protected source disk snapshot
+      that will be used to create the disk. This can be provided as a full URL
+      to the snapshot or just the snapshot name. For example, the following
+      are valid values:
+       * https://www.googleapis.com/compute/v1/projects/myproject/global/snapshots/snapshot
+       * snapshot
+      Must be specified with `source-snapshot-csek-key-file`.
+
+      *source-snapshot-csek-key-file::: Path to a Customer-Supplied Encryption
+      Key (CSEK) key file for the source snapshot. Must be specified with
+      `source-snapshot-csek-required`.
+      """
+    spec['source-snapshot-csek-key-file'] = str
+
+  if image_csek:
+    disk_help += """
+      *image-csek-required*::: Specifies the name of the CSK protected image
+      that the disk will be initialized with. A new disk will be created based
+      on the given image. To view a list of public images and projects, run
+      `$ gcloud compute images list`. It is best practice to use image when
+      a specific version of an image is needed. If both image and image-family
+      flags are omitted a blank disk will be created. Must be specified with
+      `image-csek-key-file`.
+
+      *image-csek-key-file::: Path to a Customer-Supplied Encryption Key (CSEK)
+      key file for the image. Must be specified with `image-csek-required`.
+    """
+    spec['image-csek-key-file'] = str
+
   parser.add_argument(
       '--create-disk',
       type=arg_parsers.ArgDict(spec=spec),
       action='append',
       metavar='PROPERTY=VALUE',
       help=disk_help)
-  if csek_key_file:
-    disk_help += """
-      *image-csek-key-file::: Path to a Customer-Supplied Encryption Key (CSEK)
-      key file for the image. Specified iff image is specified and the
-      image is CSEK protected.
-
-       *source-snapshot-csek-key-file::: Path to a Customer-Supplied Encryption
-       Key (CSEK) key file for the source snapshot. Specified iff
-       source-snapshot is provided and the snapshot is CSEK protected.
-    """
-    spec['csek-key-file'] = str
 
 
 def AddCustomMachineTypeArgs(parser):
@@ -678,9 +704,9 @@ def AddCustomMachineTypeArgs(parser):
       action='store_true',
       help='Use the extended custom machine type.')
   custom_group.add_argument(
-      '--custom-vm-gen',
+      '--custom-vm-type',
       help="""
-      Specifies VM generation. n1 - VMs with CPU platforms Skylake and older,
+      Specifies VM type. n1 - VMs with CPU platforms Skylake and older,
       n2 - VMs with CPU platform Cascade Lake. n2 offers flexible sizing from
       2 to 80 vCPUs, and 1 to 640GBs of memory.
       It also features a number of performance enhancements including exposing
@@ -759,12 +785,17 @@ def GetAddressRef(resources, address, region):
       })
 
 
-def ValidateDiskFlags(args, enable_kms=False, enable_snapshots=False):
+def ValidateDiskFlags(args, enable_kms=False, enable_snapshots=False,
+                      enable_source_snapshot_csek=False,
+                      enable_image_csek=False):
   """Validates the values of all disk-related flags."""
   ValidateDiskCommonFlags(args)
   ValidateDiskAccessModeFlags(args)
   ValidateDiskBootFlags(args, enable_kms=enable_kms)
-  ValidateCreateDiskFlags(args, enable_snapshots=enable_snapshots)
+  ValidateCreateDiskFlags(
+      args, enable_snapshots=enable_snapshots,
+      enable_source_snapshot_csek=enable_source_snapshot_csek,
+      enable_image_csek=enable_image_csek)
 
 
 def ValidateDiskCommonFlags(args):
@@ -871,7 +902,9 @@ def ValidateDiskBootFlags(args, enable_kms=False):
             'boot disk.')
 
 
-def ValidateCreateDiskFlags(args, enable_snapshots=False):
+def ValidateCreateDiskFlags(args, enable_snapshots=False,
+                            enable_source_snapshot_csek=False,
+                            enable_image_csek=False):
   """Validates the values of create-disk related flags."""
   require_csek_key_create = getattr(args, 'require_csek_key_create', None)
   csek_key_file = getattr(args, 'csek_key_file', None)
@@ -896,6 +929,8 @@ def ValidateCreateDiskFlags(args, enable_snapshots=False):
     image_value = disk.get('image')
     image_family_value = disk.get('image-family')
     source_snapshot = disk.get('source-snapshot')
+    image_csek_file = disk.get('image_csek')
+    source_snapshot_csek_file = disk.get('source_snapshot_csek_file')
 
     disk_source = set()
     if image_value:
@@ -904,15 +939,24 @@ def ValidateCreateDiskFlags(args, enable_snapshots=False):
       disk_source.add(image_family_value)
     if source_snapshot:
       disk_source.add(source_snapshot)
+    if image_csek_file:
+      disk_source.add(image_csek_file)
+    if source_snapshot_csek_file:
+      disk_source.add(source_snapshot_csek_file)
 
-    source_error_message = (
-        'Cannot specify [image] and [image-family] for a '
-        '[--create-disk]. The fields are mutually exclusive.')
+    mutex_attributes = ['[image]', '[image-family]']
+    if enable_image_csek:
+      mutex_attributes.append('[image-csek-required]')
     if enable_snapshots:
-      source_error_message = (
-          'Must specify exactly one of [image], [image-family] or '
-          '[source-snapshot] for a [--create-disk]. '
-          'These fields are mutually exclusive.')
+      mutex_attributes.append('[source-snapshot]')
+    if enable_source_snapshot_csek:
+      mutex_attributes.append('[source-snapshot-csek-required]')
+    formatted_attributes = '{}, or {}'.format(', '.join(mutex_attributes[:-1]),
+                                              mutex_attributes[-1])
+    source_error_message = (
+        'Must specify exactly one of {} for a '
+        '[--create-disk]. These fields are mutually exclusive.'.format(
+            formatted_attributes))
     if len(disk_source) > 1:
       raise exceptions.ToolException(source_error_message)
 
@@ -955,8 +999,8 @@ def AddAddressArgs(parser,
       'no-address': None,
       'subnet': str,
   }
-  if instances:
-    multiple_network_interface_cards_spec['private-network-ip'] = str
+
+  multiple_network_interface_cards_spec['private-network-ip'] = str
 
   def ValidateNetworkTier(network_tier_input):
     network_tier = network_tier_input.upper()
@@ -995,8 +1039,8 @@ def AddAddressArgs(parser,
         ``NETWORK_TIER'' must be one of: `PREMIUM`, `STANDARD`. The default
         value is `PREMIUM`.
         """
-    if instances:
-      network_interface_help += """
+
+    network_interface_help += """
         *private-network-ip*::: Assigns the given RFC1918 IP address to the
         interface.
         """
@@ -1085,6 +1129,24 @@ def AddMinCpuPlatformArgs(parser, track, required=False):
       """.format(track.prefix + ' ' if track.prefix else ''))
 
 
+def AddMinNodeCpuArg(parser):
+  parser.add_argument(
+      '--min-node-cpu',
+      help="""\
+      Minimum number of virtual CPUs this instance will consume when running on
+      a sole-tenant node.
+      """)
+
+
+def AddLocationHintArg(parser):
+  parser.add_argument(
+      '--location-hint',
+      hidden=True,
+      help="""\
+      Used by internal tools to control sub-zone location of the instance.
+      """)
+
+
 def AddPreemptibleVmArgs(parser):
   parser.add_argument(
       '--preemptible',
@@ -1342,7 +1404,7 @@ def ValidatePublicDnsFlags(args):
 
   network_interface = getattr(args, 'network_interface', None)
   public_dns = getattr(args, 'public_dns', None)
-  if public_dns is True:
+  if public_dns:
     if (network_interface is not None and
         network_interface != constants.DEFAULT_NETWORK_INTERFACE):
       raise exceptions.ToolException(
@@ -1356,7 +1418,7 @@ def ValidatePublicPtrFlags(args):
 
   network_interface = getattr(args, 'network_interface', None)
   public_ptr = getattr(args, 'public_ptr', None)
-  if public_ptr is True:
+  if public_ptr is True:  # pylint:disable=g-bool-id-comparison
     if (network_interface is not None and
         network_interface != constants.DEFAULT_NETWORK_INTERFACE):
       raise exceptions.ToolException(
@@ -1364,7 +1426,7 @@ def ValidatePublicPtrFlags(args):
           '\'{0}\' rather than \'{1}\'.'.format(
               constants.DEFAULT_NETWORK_INTERFACE, network_interface))
 
-  if args.public_ptr_domain is not None and args.no_public_ptr is True:
+  if args.public_ptr_domain is not None and args.no_public_ptr is True:  # pylint:disable=g-bool-id-comparison
     raise exceptions.ConflictingArgumentsException('--public-ptr-domain',
                                                    '--no-public-ptr')
 
@@ -1508,10 +1570,10 @@ def AddKonletArgs(parser):
       Each argument must have a separate flag. Arguments are appended in the
       order of flags. Example:
 
-      Assuming the default entry point of your container (or an entry point
+      Assuming the default entry point of the container (or an entry point
       overridden with --container-command flag) is a Bourne shell-compatible
       executable, in order to execute 'ls -l' command in the container,
-      you could use:
+      the user could use:
 
       `--container-arg="-c" --container-arg="ls -l"`
 
@@ -1813,13 +1875,28 @@ def AddShieldedInstanceIntegrityPolicyArgs(parser):
       help=help_text)
 
 
+def AddConfidentialComputeArgs(parser):
+  """Adds flags for confidential compute for instance."""
+  help_text = """\
+  The instance will boot with confidential compute enabled. Confidential
+  Compute is based on Secure Encrypted Virtualization (SEV), an AMD
+  virtualization feature for running confidential instances.
+  """
+  parser.add_argument(
+      '--confidential-compute',
+      dest='confidential_compute',
+      action='store_true',
+      default=None,
+      help=help_text)
+
+
 def AddHostnameArg(parser):
   """Adds flag for overriding hostname for instance."""
   parser.add_argument(
       '--hostname',
       help="""\
       Specify the hostname of the instance to be created. The specified
-      homename must be RFC1035 compliant. If hostname is not specified, the
+      hostname must be RFC1035 compliant. If hostname is not specified, the
       default hostname is [INSTANCE_NAME].c.[PROJECT_ID].internal when using
       the global DNS, and [INSTANCE_NAME].[ZONE].c.[PROJECT_ID].internal
       when using zonal DNS.
@@ -2299,3 +2376,19 @@ def AddUpdateContainerArgs(parser, container_mount_disk_enabled=False):
       parser,
       container_mount_disk_enabled=container_mount_disk_enabled)
   _AddContainerArgs(parser)
+
+
+def AddPostKeyRevocationActionTypeArgs(parser):
+  """Helper to add --post-key-revocation-action-type flag."""
+  parser.add_argument(
+      '--post-key-revocation-action-type',
+      choices=['noop', 'shutdown'],
+      metavar='POLICY',
+      required=False,
+      hidden=True,
+      help="""\
+      The instance will be shut down when the KMS key of one of its disk is
+      revoked, if set to `SHUTDOWN`.
+
+      Default setting is `NOOP`.
+      """)
