@@ -20,11 +20,11 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import contextlib
-import copy
+import datetime
 import functools
-import random
-import string
+from apitools.base.py import encoding
 from apitools.base.py import exceptions as api_exceptions
+from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.run import condition as run_condition
 from googlecloudsdk.api_lib.run import configuration
 from googlecloudsdk.api_lib.run import domain_mapping
@@ -35,34 +35,35 @@ from googlecloudsdk.api_lib.run import route
 from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import apis_internal
-from googlecloudsdk.api_lib.util import exceptions as exceptions_util
 from googlecloudsdk.api_lib.util import waiter
+from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
+from googlecloudsdk.command_lib.run import name_generator
 from googlecloudsdk.command_lib.run import resource_name_conversion
 from googlecloudsdk.command_lib.run import stages
+from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
+from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import progress_tracker
 from googlecloudsdk.core.util import retry
+import six
 
 DEFAULT_ENDPOINT_VERSION = 'v1'
 
 
-_NONCE_LENGTH = 10
-# Used to force a new revision, and also to tie a particular request for changes
-# to a particular created revision.
-NONCE_LABEL = 'client.knative.dev/nonce'
-
 # Wait 11 mins for each deployment. This is longer than the server timeout,
 # making it more likely to get a useful error message from the server.
 MAX_WAIT_MS = 660000
 
-ALLOW_UNAUTH_POLICY_BINDING_MEMBERS = ['allUsers']
+ALLOW_UNAUTH_POLICY_BINDING_MEMBER = 'allUsers'
 ALLOW_UNAUTH_POLICY_BINDING_ROLE = 'roles/run.invoker'
 
+NEEDED_IAM_PERMISSIONS = ['run.services.setIamPolicy']
+
 
 class UnknownAPIError(exceptions.Error):
   pass
@@ -89,10 +90,9 @@ def Connect(conn_context):
   # connection context so that it does not pick up the api_endpoint_overrides
   # values from the connection context.
   # pylint: disable=protected-access
-  op_client = apis_internal._GetClientInstance(
+  op_client = apis.GetClientInstance(
       conn_context.api_name,
-      conn_context.api_version,
-      ca_certs=conn_context.ca_certs)
+      conn_context.api_version)
   # pylint: enable=protected-access
 
   with conn_context as conn_info:
@@ -101,9 +101,9 @@ def Connect(conn_context):
         conn_info.api_name,
         conn_info.api_version,
         # Only check response if not connecting to GKE
-        check_response_func=apis.CheckResponseForApiEnablement
+        check_response_func=apis.CheckResponseForApiEnablement()
         if conn_context.supports_one_platform else None,
-        ca_certs=conn_info.ca_certs)
+        http_client=conn_context.HttpClient())
     # pylint: enable=protected-access
     yield ServerlessOperations(
         client,
@@ -143,7 +143,11 @@ class ConditionPoller(waiter.OperationPoller):
   Takes in a reference to a StagedProgressTracker, and updates it with progress.
   """
 
-  def __init__(self, resource_getter, tracker, dependencies=None):
+  def __init__(self,
+               resource_getter,
+               tracker,
+               dependencies=None,
+               ready_message='Done.'):
     """Initialize the ConditionPoller.
 
     Start any unblocked stages in the tracker immediately.
@@ -167,19 +171,29 @@ class ConditionPoller(waiter.OperationPoller):
 
         If the entire dependencies dict is None, the poller will assume that
         all keys in the tracker are relevant and none have dependencies.
+
+      ready_message: str, message to display in header of tracker when
+        conditions are ready.
     """
     # _dependencies is a map of condition -> {preceding conditions}
     # It is meant to be checked off as we finish things.
     self._dependencies = {k: set() for k in tracker}
     if dependencies is not None:
       for k in dependencies:
+        error_msg = 'Condition {} is not in the dependency tracker.'
         if k not in tracker:
-          raise ValueError(
-              'Condition {} is not in the dependency tracker.'.format(k))
-        self._dependencies[k] = copy.copy(dependencies[k])
+          raise ValueError(error_msg.format(k))
+        for c in dependencies[k]:
+          if c not in tracker:
+            raise ValueError(error_msg.format(c))
+        # Add dependencies, only if they're still not complete
+        self._dependencies[k] = {
+            c for c in dependencies[k] if not tracker.IsComplete(c)
+        }
     self._resource_getter = resource_getter
     self._tracker = tracker
     self._resource_fail_type = exceptions.Error
+    self._ready_message = ready_message
     self._StartUnblocked()
 
   def _IsBlocked(self, condition):
@@ -198,13 +212,13 @@ class ConditionPoller(waiter.OperationPoller):
       return False
     return conditions.IsTerminal()
 
-  def _PollTerminalSubconditions(self, conditions, ready_message):
+  def _PollTerminalSubconditions(self, conditions, conditions_message):
     for condition in conditions.TerminalSubconditions():
       if condition not in self._dependencies:
         continue
       message = conditions[condition]['message']
       status = conditions[condition]['status']
-      self._PossiblyUpdateMessage(condition, message, ready_message)
+      self._PossiblyUpdateMessage(condition, message, conditions_message)
       if status is None:
         continue
       elif status:
@@ -213,7 +227,7 @@ class ConditionPoller(waiter.OperationPoller):
           # Check all terminal subconditions again to ensure any stages that
           # were unblocked by this stage completing are re-checked before we
           # check the ready condition
-          self._PollTerminalSubconditions(conditions, ready_message)
+          self._PollTerminalSubconditions(conditions, conditions_message)
           break
       else:
         self._PossiblyFailStage(condition, message)
@@ -233,28 +247,29 @@ class ConditionPoller(waiter.OperationPoller):
     if conditions is None or not conditions.IsFresh():
       return None
 
-    ready_message = conditions.DescriptiveMessage()
-    if ready_message:
-      self._tracker.UpdateHeaderMessage(ready_message)
+    conditions_message = conditions.DescriptiveMessage()
+    if conditions_message:
+      self._tracker.UpdateHeaderMessage(conditions_message)
 
-    self._PollTerminalSubconditions(conditions, ready_message)
+    self._PollTerminalSubconditions(conditions, conditions_message)
 
     if conditions.IsReady():
-      self._tracker.UpdateHeaderMessage('Done.')
+      self._tracker.UpdateHeaderMessage(self._ready_message)
       # TODO(b/120679874): Should not have to manually call Tick()
       self._tracker.Tick()
     elif conditions.IsFailed():
-      raise self._resource_fail_type(ready_message)
+      raise self._resource_fail_type(conditions_message)
 
     return conditions
 
-  def _PossiblyUpdateMessage(self, condition, message, ready_message):
+  def _PossiblyUpdateMessage(self, condition, message, conditions_message):
     """Update the stage message.
 
     Args:
       condition: str, The name of the status condition.
       message: str, The new message to display
-      ready_message: str, The ready message we're displaying.
+      conditions_message: str, The message from the conditions object we're
+        displaying..
     """
 
     if self._tracker.IsComplete(condition):
@@ -263,7 +278,7 @@ class ConditionPoller(waiter.OperationPoller):
     if self._IsBlocked(condition):
       return
 
-    if message != ready_message:
+    if message != conditions_message:
       self._tracker.UpdateStage(condition, message)
 
   def _RecordConditionComplete(self, condition):
@@ -355,33 +370,59 @@ class ConditionPoller(waiter.OperationPoller):
       A condition.Conditions object.
     """
     resource = self._resource_getter()
+
     if resource is None:
       return None
     return resource.conditions
 
 
 class ServiceConditionPoller(ConditionPoller):
+  """A ConditionPoller for services."""
+
+  def __init__(self, getter, tracker, dependencies=None, serv=None):
+    def GetIfProbablyNewer():
+      """Workaround for https://github.com/knative/serving/issues/4149).
+
+      The workaround is to wait for the condition lastTransitionTime to
+      change. Note that the granularity of lastTransitionTime is seconds
+      so to avoid hanging in the unlikely case that two updates happen
+      in the same second we limit waiting for the change to 5 seconds.
+
+      Returns:
+        The requested resource or None if it seems stale.
+      """
+      resource = getter()
+      if (resource
+          and self._old_last_transition_time
+          and self._old_last_transition_time == resource.last_transition_time
+          and not self.HaveFiveSecondsPassed()):
+        return None
+      else:
+        return resource
 
-  def __init__(self, getter, tracker):
     super(ServiceConditionPoller, self).__init__(
-        getter, tracker, stages.ServiceDependencies())
+        GetIfProbablyNewer, tracker, dependencies)
     self._resource_fail_type = serverless_exceptions.DeploymentFailedError
+    self._old_last_transition_time = serv.last_transition_time if serv else None
+    self._start_time = datetime.datetime.now()
+    self._five_seconds = datetime.timedelta(seconds=5)
 
-
-def _Nonce():
-  """Return a random string with unlikely collision to use as a nonce."""
-  return ''.join(
-      random.choice(string.ascii_lowercase) for _ in range(_NONCE_LENGTH))
+  def HaveFiveSecondsPassed(self):
+    return datetime.datetime.now() - self._start_time > self._five_seconds
 
 
 class _NewRevisionForcingChange(config_changes_mod.ConfigChanger):
-  """Forces a new revision to get created by posting a random nonce label."""
+  """Forces a new revision to get created by changing the revision name."""
 
-  def __init__(self, nonce):
-    self._nonce = nonce
+  def __init__(self, revision_suffix):
+    self._revision_name_change = config_changes_mod.RevisionNameChanges(
+        revision_suffix)
 
   def Adjust(self, resource):
-    resource.template.labels[NONCE_LABEL] = self._nonce
+    """Adjust by revision name."""
+    if revision.NONCE_LABEL in resource.template.labels:
+      del resource.template.labels[revision.NONCE_LABEL]
+    return self._revision_name_change.Adjust(resource)
 
 
 def _IsDigest(url):
@@ -389,6 +430,24 @@ def _IsDigest(url):
   return '@sha256:' in url
 
 
+class RevisionNameBasedPoller(waiter.OperationPoller):
+  """Poll for the revision with the given name to exist."""
+
+  def __init__(self, operations, revision_ref_getter):
+    self._operations = operations
+    self._revision_ref_getter = revision_ref_getter
+
+  def IsDone(self, revision_obj):
+    return bool(revision_obj)
+
+  def Poll(self, revision_name):
+    revision_ref = self._revision_ref_getter(revision_name)
+    return self._operations.GetRevision(revision_ref)
+
+  def GetResult(self, revision_obj):
+    return revision_obj
+
+
 class NonceBasedRevisionPoller(waiter.OperationPoller):
   """To poll for exactly one revision with the given nonce to appear."""
 
@@ -416,14 +475,33 @@ class _SwitchToDigestChange(config_changes_mod.ConfigChanger):
 
   def Adjust(self, resource):
     if _IsDigest(self._base_revision.image):
-      return
+      return resource
     if not self._base_revision.image_digest:
-      return
+      return resource
 
     # Mutates through to metadata: Save the by-tag user intent.
-    resource.annotations[configuration.USER_IMAGE_ANNOTATION] = (
+    resource.annotations[revision.USER_IMAGE_ANNOTATION] = (
+        self._base_revision.image)
+    resource.template.annotations[revision.USER_IMAGE_ANNOTATION] = (
         self._base_revision.image)
     resource.template.image = self._base_revision.image_digest
+    return resource
+
+_CLIENT_NAME_ANNOTATION = 'run.googleapis.com/client-name'
+_CLIENT_NAME = 'gcloud'
+_CLIENT_VERSION_ANNOTATION = 'run.googleapis.com/client-version'
+
+
+class _SetClientNameAndVersion(config_changes_mod.ConfigChanger):
+  """Sets the client name and version annotations."""
+
+  def Adjust(self, resource):
+    resource.annotations[_CLIENT_NAME_ANNOTATION] = _CLIENT_NAME
+    resource.template.annotations[_CLIENT_NAME_ANNOTATION] = _CLIENT_NAME
+    resource.annotations[_CLIENT_VERSION_ANNOTATION] = config.CLOUD_SDK_VERSION
+    resource.template.annotations[
+        _CLIENT_VERSION_ANNOTATION] = config.CLOUD_SDK_VERSION
+    return resource
 
 
 class ServerlessOperations(object):
@@ -449,7 +527,7 @@ class ServerlessOperations(object):
     self._region = region
 
   @property
-  def _messages_module(self):
+  def messages_module(self):
     return self._client.MESSAGES_MODULE
 
   def GetRevision(self, revision_ref):
@@ -461,7 +539,7 @@ class ServerlessOperations(object):
     Returns:
       A revision.Revision object.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     revision_name = revision_ref.RelativeName()
     request = messages.RunNamespacesRevisionsGetRequest(
         name=revision_name)
@@ -478,7 +556,7 @@ class ServerlessOperations(object):
 
   def _GetRoute(self, service_ref):
     """Return the relevant Route from the server, or None if 404."""
-    messages = self._messages_module
+    messages = self.messages_module
     # GET the Route
     route_name = self._registry.Parse(
         service_ref.servicesId,
@@ -526,15 +604,6 @@ class ServerlessOperations(object):
       raise serverless_exceptions.ConfigurationError(
           conditions.DescriptiveMessage())
 
-  def GetServiceUrl(self, service_ref):
-    """Return the main URL for the service."""
-    serv = self.GetService(service_ref)
-    if serv.domain:
-      return serv.domain
-    # Older versions of knative don't populate domain on Service, only Route.
-    serv_route = self._GetRoute(service_ref)
-    return serv_route.domain
-
   def GetActiveRevisions(self, service_ref):
     """Return the actively serving revisions.
 
@@ -557,7 +626,7 @@ class ServerlessOperations(object):
     return serv_route.active_revisions
 
   def ListServices(self, namespace_ref):
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesServicesListRequest(
         parent=namespace_ref.RelativeName())
     with metrics.RecordDuration(metric_names.LIST_SERVICES):
@@ -565,7 +634,7 @@ class ServerlessOperations(object):
     return [service.Service(item, messages) for item in response.items]
 
   def ListConfigurations(self, namespace_ref):
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesConfigurationsListRequest(
         parent=namespace_ref.RelativeName())
     with metrics.RecordDuration(metric_names.LIST_CONFIGURATIONS):
@@ -574,7 +643,7 @@ class ServerlessOperations(object):
             for item in response.items]
 
   def ListRoutes(self, namespace_ref):
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesRoutesListRequest(
         parent=namespace_ref.RelativeName())
     with metrics.RecordDuration(metric_names.LIST_ROUTES):
@@ -583,7 +652,7 @@ class ServerlessOperations(object):
 
   def GetService(self, service_ref):
     """Return the relevant Service from the server, or None if 404."""
-    messages = self._messages_module
+    messages = self.messages_module
     service_get_request = messages.RunNamespacesServicesGetRequest(
         name=service_ref.RelativeName())
 
@@ -597,7 +666,7 @@ class ServerlessOperations(object):
 
   def GetConfiguration(self, service_or_configuration_ref):
     """Return the relevant Configuration from the server, or None if 404."""
-    messages = self._messages_module
+    messages = self.messages_module
     if hasattr(service_or_configuration_ref, 'servicesId'):
       name = self._registry.Parse(
           service_or_configuration_ref.servicesId,
@@ -621,7 +690,7 @@ class ServerlessOperations(object):
 
   def GetRoute(self, service_or_route_ref):
     """Return the relevant Route from the server, or None if 404."""
-    messages = self._messages_module
+    messages = self.messages_module
     if hasattr(service_or_route_ref, 'servicesId'):
       name = self._registry.Parse(
           service_or_route_ref.servicesId,
@@ -652,7 +721,7 @@ class ServerlessOperations(object):
     Raises:
       ServiceNotFoundError: if provided service is not found.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     service_name = service_ref.RelativeName()
     service_delete_request = messages.RunNamespacesServicesDeleteRequest(
         name=service_name,
@@ -674,7 +743,7 @@ class ServerlessOperations(object):
     Raises:
       RevisionNotFoundError: if provided revision is not found.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     revision_name = revision_ref.RelativeName()
     request = messages.RunNamespacesRevisionsDeleteRequest(
         name=revision_name)
@@ -687,10 +756,10 @@ class ServerlessOperations(object):
 
   def GetRevisionsByNonce(self, namespace_ref, nonce):
     """Return all revisions with the given nonce."""
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesRevisionsListRequest(
         parent=namespace_ref.RelativeName(),
-        labelSelector='{} = {}'.format(NONCE_LABEL, nonce))
+        labelSelector='{} = {}'.format(revision.NONCE_LABEL, nonce))
     response = self._client.namespaces_revisions.List(request)
     return [revision.Revision(item, messages) for item in response.items]
 
@@ -702,8 +771,9 @@ class ServerlessOperations(object):
     we get the digest for the image to run.
 
     Getting this revision:
+      * If there's a name in the template metadata, use that
       * If there's a nonce in the revisonTemplate metadata, use that
-      * If that query produces >1 or produces 0 after a short timeout, use
+      * If that query produces >1 or 0 after a short timeout, use
         the latestCreatedRevision in status.
 
     Arguments:
@@ -714,23 +784,39 @@ class ServerlessOperations(object):
         level object.
 
     Returns:
-      The base revision of the configuration.
+      The base revision of the configuration or None if not found by revision
+        name nor nonce and latestCreatedRevisionName does not exist on the
+        Service object.
     """
-    # Or returns None if not available by nonce & the control plane has not
-    # implemented latestCreatedRevisionName on the Service object yet.
-    base_revision_nonce = template.labels.get(NONCE_LABEL, None)
     base_revision = None
-    if base_revision_nonce:
+    # Try to find by revision name
+    base_revision_name = template.name
+    if base_revision_name:
       try:
-        namespace_ref = self._registry.Parse(
-            metadata.namespace,
-            collection='run.namespaces')
-        poller = NonceBasedRevisionPoller(self, namespace_ref)
-        base_revision = poller.GetResult(waiter.PollUntilDone(
-            poller, base_revision_nonce,
-            sleep_ms=500, max_wait_ms=2000))
-      except retry.WaitException:
+        revision_ref_getter = functools.partial(
+            self._registry.Parse,
+            params={'namespacesId': metadata.namespace},
+            collection='run.namespaces.revisions')
+        poller = RevisionNameBasedPoller(self, revision_ref_getter)
+        base_revision = poller.GetResult(
+            waiter.PollUntilDone(
+                poller, base_revision_name, sleep_ms=500, max_wait_ms=2000))
+      except retry.RetryException:
         pass
+    # Name polling didn't work. Fall back to nonce polling
+    if not base_revision:
+      base_revision_nonce = template.labels.get(revision.NONCE_LABEL, None)
+      if base_revision_nonce:
+        try:
+          namespace_ref = self._registry.Parse(
+              metadata.namespace,
+              collection='run.namespaces')
+          poller = NonceBasedRevisionPoller(self, namespace_ref)
+          base_revision = poller.GetResult(waiter.PollUntilDone(
+              poller, base_revision_nonce,
+              sleep_ms=500, max_wait_ms=2000))
+        except retry.RetryException:
+          pass
     # Nonce polling didn't work, because some client didn't post one or didn't
     # change one. Fall back to the (slightly racy) `latestCreatedRevisionName`.
     if not base_revision:
@@ -752,8 +838,8 @@ class ServerlessOperations(object):
       if base_revision:
         config_changes.append(_SwitchToDigestChange(base_revision))
 
-  def _UpdateOrCreateService(self, service_ref, config_changes, with_code,
-                             private_endpoint=None):
+  def _UpdateOrCreateService(
+      self, service_ref, config_changes, with_code, serv):
     """Apply config_changes to the service. Create it if necessary.
 
     Arguments:
@@ -761,41 +847,18 @@ class ServerlessOperations(object):
       config_changes: list of ConfigChanger to modify the service with
       with_code: bool, True if the config_changes contains code to deploy.
         We can't create the service if we're not deploying code.
-      private_endpoint: bool, True if creating a new Service for
-        Cloud Run on GKE that should only be addressable from within the
-        cluster. False if it should be publicly addressable. None if
-        its existing visibility should remain unchanged.
+      serv: service.Service, For update the Service to update and for
+        create None.
 
     Returns:
       The Service object we created or modified.
     """
-    nonce = _Nonce()
-    config_changes = [_NewRevisionForcingChange(nonce)] + config_changes
-    messages = self._messages_module
-    # GET the Service
-    serv = self.GetService(service_ref)
+    messages = self.messages_module
     try:
       if serv:
-        if not with_code:
-          # Avoid changing the running code by making the new revision by digest
-          self._EnsureImageDigest(serv, config_changes)
-
-        if private_endpoint is None:
-          # Don't change the existing service visibility
-          pass
-        elif private_endpoint:
-          serv.labels[service.ENDPOINT_VISIBILITY] = service.CLUSTER_LOCAL
-        elif service.ENDPOINT_VISIBILITY in serv.labels:
-          del serv.labels[service.ENDPOINT_VISIBILITY]
-
-        # Revision names must be unique across the namespace.
-        # To prevent the revision name being unchanged from the last revision,
-        # we reset the value so the default naming scheme will be used instead.
-        serv.template.name = None
-
         # PUT the changed Service
         for config_change in config_changes:
-          config_change.Adjust(serv)
+          serv = config_change.Adjust(serv)
         serv_name = service_ref.RelativeName()
         serv_update_req = (
             messages.RunNamespacesServicesReplaceServiceRequest(
@@ -811,12 +874,11 @@ class ServerlessOperations(object):
           raise serverless_exceptions.ServiceNotFoundError(
               'Service [{}] could not be found.'.format(service_ref.servicesId))
         # POST a new Service
-        new_serv = service.Service.New(self._client, service_ref.namespacesId,
-                                       private_endpoint)
+        new_serv = service.Service.New(self._client, service_ref.namespacesId)
         new_serv.name = service_ref.servicesId
         parent = service_ref.Parent().RelativeName()
         for config_change in config_changes:
-          config_change.Adjust(new_serv)
+          new_serv = config_change.Adjust(new_serv)
         serv_create_req = (
             messages.RunNamespacesServicesCreateRequest(
                 service=new_serv.Message(),
@@ -826,15 +888,11 @@ class ServerlessOperations(object):
               serv_create_req)
         return service.Service(raw_service, messages)
     except api_exceptions.HttpBadRequestError as e:
-      error_payload = exceptions_util.HttpErrorPayload(e)
-      if error_payload.field_violations:
-        if (serverless_exceptions.BadImageError.IMAGE_ERROR_FIELD
-            in error_payload.field_violations):
-          exceptions.reraise(serverless_exceptions.BadImageError(e))
-      exceptions.reraise(e)
+      exceptions.reraise(serverless_exceptions.HttpError(e))
     except api_exceptions.HttpNotFoundError as e:
+      platform = properties.VALUES.run.platform.Get()
       error_msg = 'Deployment endpoint was not found.'
-      if not self._region:
+      if platform == 'gke':
         all_clusters = global_methods.ListClusters()
         clusters = ['* {} in {}'.format(c.name, c.zone) for c in all_clusters]
         error_msg += (' Perhaps the provided cluster was invalid or '
@@ -843,7 +901,7 @@ class ServerlessOperations(object):
                       '`run/cluster` and `run/cluster_location` properties to '
                       'a valid cluster and zone and retry.'
                       '\nAvailable clusters:\n{}'.format('\n'.join(clusters)))
-      else:
+      elif platform == 'managed':
         all_regions = global_methods.ListRegions(self._op_client)
         if self._region not in all_regions:
           regions = ['* {}'.format(r) for r in all_regions]
@@ -851,11 +909,57 @@ class ServerlessOperations(object):
                         'Pass the `--region` flag or set the '
                         '`run/region` property to a valid region and retry.'
                         '\nAvailable regions:\n{}'.format('\n'.join(regions)))
+      elif platform == 'kubernetes':
+        error_msg += (' Perhaps the provided cluster was invalid or '
+                      'does not have Cloud Run enabled. Ensure in your '
+                      'kubeconfig file that the cluster referenced in '
+                      'the current context or the specified context '
+                      'is a valid cluster and retry.')
       raise serverless_exceptions.DeploymentFailedError(error_msg)
+    except api_exceptions.HttpError as e:
+      platform = properties.VALUES.run.platform.Get()
+      if platform == 'managed':
+        exceptions.reraise(e)
+      k8s_error = serverless_exceptions.KubernetesExceptionParser(e)
+      causes = '\n\n'.join([c['message'] for c in k8s_error.causes])
+      if not causes:
+        causes = k8s_error.error
+      raise serverless_exceptions.KubernetesError('Error{}:\n{}\n'.format(
+          's' if len(k8s_error.causes) > 1 else '', causes))
+
+  def UpdateTraffic(self, service_ref, config_changes, tracker, asyn):
+    """Update traffic splits for service."""
+    if tracker is None:
+      tracker = progress_tracker.NoOpStagedProgressTracker(
+          stages.UpdateTrafficStages(),
+          interruptable=True,
+          aborted_message='aborted')
+    serv = self.GetService(service_ref)
+    if not serv:
+      raise serverless_exceptions.ServiceNotFoundError(
+          'Service [{}] could not be found.'.format(service_ref.servicesId))
+
+    if serv.configuration:
+      raise serverless_exceptions.UnsupportedOperationError(
+          'This service is using an old version of Cloud Run for Anthos '
+          'that does not support traffic features. Please upgrade to 0.8 '
+          'or later.')
+
+    self._UpdateOrCreateService(service_ref, config_changes, False, serv)
+
+    if not asyn:
+      getter = functools.partial(self.GetService, service_ref)
+      self.WaitForCondition(ServiceConditionPoller(getter, tracker, serv=serv))
+
+  def _AddRevisionForcingChange(self, serv, config_changes):
+    """Get a new revision forcing config change for the given service."""
+    curr_generation = serv.generation if serv is not None else 0
+    revision_suffix = '{}-{}'.format(
+        str(curr_generation + 1).zfill(5), name_generator.GenerateName())
+    config_changes.insert(0, _NewRevisionForcingChange(revision_suffix))
 
   def ReleaseService(self, service_ref, config_changes, tracker=None,
-                     asyn=False, private_endpoint=None,
-                     allow_unauthenticated=None):
+                     asyn=False, allow_unauthenticated=None, for_replace=False):
     """Change the given service in prod using the given config_changes.
 
     Ensures a new revision is always created, even if the spec of the revision
@@ -865,31 +969,39 @@ class ServerlessOperations(object):
       service_ref: Resource, the service to release
       config_changes: list, objects that implement Adjust().
       tracker: StagedProgressTracker, to report on the progress of releasing.
-      asyn: bool, if True, release asyncronously
-      private_endpoint: bool, True if creating a new Service for
-        Cloud Run on GKE that should only be addressable from within the
-        cluster. False if it should be publicly addressable. None if
-        its existing visibility should remain unchanged.
+      asyn: bool, if True, return without waiting for the service to be updated.
       allow_unauthenticated: bool, True if creating a hosted Cloud Run
         service which should also have its IAM policy set to allow
         unauthenticated access. False if removing the IAM policy to allow
         unauthenticated access from a service.
+      for_replace: bool, If the change is for a replacing the service from a
+        YAML specification.
     """
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
           stages.ServiceStages(allow_unauthenticated is not None),
           interruptable=True, aborted_message='aborted')
-    with_image = any(
-        isinstance(c, config_changes_mod.ImageChange) for c in config_changes)
+    serv = self.GetService(service_ref)
+    if for_replace:
+      with_image = True
+    else:
+      with_image = any(
+          isinstance(c, config_changes_mod.ImageChange) for c in config_changes)
+      self._AddRevisionForcingChange(serv, config_changes)
+      if serv and not with_image:
+        # Avoid changing the running code by making the new revision by digest
+        self._EnsureImageDigest(serv, config_changes)
+    config_changes = [_SetClientNameAndVersion()] + config_changes
+
     self._UpdateOrCreateService(
-        service_ref, config_changes, with_image, private_endpoint)
+        service_ref, config_changes, with_image, serv)
 
     if allow_unauthenticated is not None:
       try:
         tracker.StartStage(stages.SERVICE_IAM_POLICY_SET)
         tracker.UpdateStage(stages.SERVICE_IAM_POLICY_SET, '')
         self.AddOrRemoveIamPolicyBinding(service_ref, allow_unauthenticated,
-                                         ALLOW_UNAUTH_POLICY_BINDING_MEMBERS,
+                                         ALLOW_UNAUTH_POLICY_BINDING_MEMBER,
                                          ALLOW_UNAUTH_POLICY_BINDING_ROLE)
         tracker.CompleteStage(stages.SERVICE_IAM_POLICY_SET)
       except api_exceptions.HttpError:
@@ -905,12 +1017,17 @@ class ServerlessOperations(object):
 
     if not asyn:
       getter = functools.partial(self.GetService, service_ref)
-      poller = ServiceConditionPoller(getter, tracker)
+      poller = ServiceConditionPoller(
+          getter,
+          tracker,
+          dependencies=stages.ServiceDependencies(),
+          serv=serv)
       self.WaitForCondition(poller)
       for msg in run_condition.GetNonTerminalMessages(poller.GetConditions()):
         tracker.AddWarning(msg)
 
-  def ListRevisions(self, namespace_ref, service_name):
+  def ListRevisions(self, namespace_ref, service_name,
+                    limit=None, page_size=100):
     """List all revisions for the given service.
 
     Revision list gets sorted by service name and creation timestamp.
@@ -918,11 +1035,17 @@ class ServerlessOperations(object):
     Args:
       namespace_ref: Resource, namespace to list revisions in
       service_name: str, The service for which to list revisions.
+      limit: Optional[int], max number of revisions to list.
+      page_size: Optional[int], number of revisions to fetch at a time
 
-    Returns:
-      A list of revisions for the given service.
+    Yields:
+      Revisions for the given surface
     """
-    messages = self._messages_module
+    messages = self.messages_module
+    # NB: This is a hack to compensate for apitools not generating this line.
+    #     It's necessary to make the URL parameter be "continue".
+    encoding.AddCustomJsonFieldMapping(
+        messages.RunNamespacesRevisionsListRequest, 'continue_', 'continue')
     request = messages.RunNamespacesRevisionsListRequest(
         parent=namespace_ref.RelativeName(),
     )
@@ -931,15 +1054,15 @@ class ServerlessOperations(object):
       # 'service-less' operation.
       request.labelSelector = 'serving.knative.dev/service = {}'.format(
           service_name)
-    with metrics.RecordDuration(metric_names.LIST_REVISIONS):
-      response = self._client.namespaces_revisions.List(request)
-
-    # Server does not sort the response so we'll need to sort client-side
-    revisions = [revision.Revision(item, messages) for item in response.items]
-    # Newest first
-    revisions.sort(key=lambda r: r.creation_timestamp, reverse=True)
-    revisions.sort(key=lambda r: r.service_name)
-    return revisions
+    for result in list_pager.YieldFromList(
+        service=self._client.namespaces_revisions,
+        request=request,
+        limit=limit,
+        batch_size=page_size,
+        current_token_attribute='continue_',
+        next_token_attribute=('metadata', 'continue_'),
+        batch_size_attribute='limit'):
+      yield revision.Revision(result, messages)
 
   def ListDomainMappings(self, namespace_ref):
     """List all domain mappings.
@@ -950,7 +1073,7 @@ class ServerlessOperations(object):
     Returns:
       A list of domain mappings.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesDomainmappingsListRequest(
         parent=namespace_ref.RelativeName())
     with metrics.RecordDuration(metric_names.LIST_DOMAIN_MAPPINGS):
@@ -969,7 +1092,7 @@ class ServerlessOperations(object):
       A domain_mapping.DomainMapping object.
     """
 
-    messages = self._messages_module
+    messages = self.messages_module
     new_mapping = domain_mapping.DomainMapping.New(
         self._client, domain_mapping_ref.namespacesId)
     new_mapping.name = domain_mapping_ref.domainmappingsId
@@ -990,16 +1113,15 @@ class ServerlessOperations(object):
         mapping = waiter.PollUntilDone(
             DomainMappingResourceRecordPoller(self), domain_mapping_ref)
       ready = mapping.conditions.get('Ready')
-      records = getattr(mapping.status, 'resourceRecords', None)
       message = None
       if ready and ready.get('message'):
         message = ready['message']
-      if not records:
+      if not mapping.records:
         raise serverless_exceptions.DomainMappingCreationError(
             message or 'Could not create domain mapping.')
       if message:
         log.status.Print(message)
-      return records
+      return mapping
 
     return domain_mapping.DomainMapping(response, messages)
 
@@ -1009,7 +1131,7 @@ class ServerlessOperations(object):
     Args:
       domain_mapping_ref: Resource, domainmapping resource.
     """
-    messages = self._messages_module
+    messages = self.messages_module
 
     request = messages.RunNamespacesDomainmappingsDeleteRequest(
         name=domain_mapping_ref.RelativeName())
@@ -1025,7 +1147,7 @@ class ServerlessOperations(object):
     Returns:
       A domain_mapping.DomainMapping object.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunNamespacesDomainmappingsGetRequest(
         name=domain_mapping_ref.RelativeName())
     with metrics.RecordDuration(metric_names.GET_DOMAIN_MAPPING):
@@ -1034,14 +1156,14 @@ class ServerlessOperations(object):
 
   def _GetIamPolicy(self, service_name):
     """Gets the IAM policy for the service."""
-    messages = self._messages_module
+    messages = self.messages_module
     request = messages.RunProjectsLocationsServicesGetIamPolicyRequest(
-        resource=str(service_name))
+        resource=six.text_type(service_name))
     response = self._op_client.projects_locations_services.GetIamPolicy(request)
     return response
 
   def AddOrRemoveIamPolicyBinding(self, service_ref, add_binding=True,
-                                  members=None, role=None):
+                                  member=None, role=None):
     """Add or remove the given IAM policy binding to the provided service.
 
     If no members or role are provided, set the IAM policy to the current IAM
@@ -1051,33 +1173,37 @@ class ServerlessOperations(object):
     Args:
       service_ref: str, The service to which to add the IAM policy.
       add_binding: bool, Whether to add to or remove from the IAM policy.
-      members: [str], The users for which the binding applies.
+      member: str, One of the users for which the binding applies.
       role: str, The role to grant the provided members.
 
     Returns:
       A google.iam.v1.TestIamPermissionsResponse.
     """
-    messages = self._messages_module
+    messages = self.messages_module
     oneplatform_service = resource_name_conversion.K8sToOnePlatform(
         service_ref, self._region)
     policy = self._GetIamPolicy(oneplatform_service)
-    # Don't modify bindings if not members or roles provided
-    if members and role:
-      binding = messages.Binding(members=members, role=role)
+    # Don't modify bindings if not member or roles provided
+    if member and role:
       if add_binding:
-        policy.bindings.append(binding)
-      else:
-        # Remove bindings that exactly match the provided members and role
-        policy.bindings = [b for b in policy.bindings if b != binding]
+        iam_util.AddBindingToIamPolicy(messages.Binding, policy, member, role)
+      elif iam_util.BindingInPolicy(policy, member, role):
+        iam_util.RemoveBindingFromIamPolicy(policy, member, role)
     request = messages.RunProjectsLocationsServicesSetIamPolicyRequest(
-        resource=str(oneplatform_service),
+        resource=six.text_type(oneplatform_service),
         setIamPolicyRequest=messages.SetIamPolicyRequest(policy=policy))
     result = self._op_client.projects_locations_services.SetIamPolicy(request)
     return result
 
   def CanSetIamPolicyBinding(self, service_ref):
-    try:
-      self.AddOrRemoveIamPolicyBinding(service_ref)
-      return True
-    except api_exceptions.HttpError:
-      return False
+    """Check if user has permission to set the iam policy on the service."""
+    messages = self.messages_module
+    oneplatform_service = resource_name_conversion.K8sToOnePlatform(
+        service_ref, self._region)
+    request = messages.RunProjectsLocationsServicesTestIamPermissionsRequest(
+        resource=six.text_type(oneplatform_service),
+        testIamPermissionsRequest=messages.TestIamPermissionsRequest(
+            permissions=NEEDED_IAM_PERMISSIONS))
+    response = self._op_client.projects_locations_services.TestIamPermissions(
+        request)
+    return set(NEEDED_IAM_PERMISSIONS).issubset(set(response.permissions))
