@@ -320,7 +320,7 @@ class CreateBeta(Create):
   """Create a forwarding rule to direct network traffic to a load balancer."""
   _support_global_access = True
   _support_l7_internal_load_balancing = True
-  _support_mirroring_collector = False
+  _support_mirroring_collector = True
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -332,26 +332,26 @@ class CreateAlpha(CreateBeta):
 
 
 Create.detailed_help = {
-    'DESCRIPTION': ("""\
-        *{{command}}* is used to create a forwarding rule. {overview}
+    'DESCRIPTION': ("""
+*{{command}}* is used to create a forwarding rule. {overview}
 
-        When creating a forwarding rule, exactly one of  ``--target-instance'',
-        ``--target-pool'', ``--target-http-proxy'', ``--target-https-proxy'',
-        ``--target-ssl-proxy'', ``--target-tcp-proxy'', ``--target-vpn-gateway''
-        or ``--backend-service'' must be specified."""
+When creating a forwarding rule, exactly one of  ``--target-instance'',
+``--target-pool'', ``--target-http-proxy'', ``--target-https-proxy'',
+``--target-ssl-proxy'', ``--target-tcp-proxy'', ``--target-vpn-gateway''
+or ``--backend-service'' must be specified."""
                     .format(overview=flags.FORWARDING_RULES_OVERVIEW)),
-    'EXAMPLES': """\
-        To create a global forwarding rule that will forward all traffic on port
-        8080 for IP address ADDRESS to a target http proxy PROXY, run:
+    'EXAMPLES': """
+    To create a global forwarding rule that will forward all traffic on port
+    8080 for IP address ADDRESS to a target http proxy PROXY, run:
 
-          $ {command} RULE_NAME --global --target-http-proxy PROXY --ports 8080 --address ADDRESS
+      $ {command} RULE_NAME --global --target-http-proxy=PROXY --ports=8080 --address=ADDRESS
 
-        To create a regional forwarding rule for the subnet SUBNET_NAME on the
-        default network that will forward all traffic on ports 80-82 to a
-        backend service SERVICE_NAME, run:
+    To create a regional forwarding rule for the subnet SUBNET_NAME on the
+    default network that will forward all traffic on ports 80-82 to a
+    backend service SERVICE_NAME, run:
 
-          $ {command} RULE_NAME --load-balancing-scheme INTERNAL --backend-service SERVICE_NAME --subnet SUBNET_NAME --network default --region REGION --ports 80-82
-    """
+      $ {command} RULE_NAME --load-balancing-scheme=INTERNAL --backend-service=SERVICE_NAME --subnet=SUBNET_NAME --network=default --region=REGION --ports=80-82
+"""
 }
 
 
