@@ -69,7 +69,7 @@ def _AddCommonArgs(parser):
 
 def _AddMutuallyExclusiveArgs(mutex_group, release_track):
   """Add all arguments that need to be mutually exclusive from each other."""
-  if release_track in [base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA]:
+  if release_track == base.ReleaseTrack.ALPHA:
     mutex_group.add_argument(
         '--update-addons',
         type=arg_parsers.ArgDict(
@@ -80,6 +80,8 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
                 api_adapter.NETWORK_POLICY: _ParseAddonDisabled,
                 api_adapter.ISTIO: _ParseAddonDisabled,
                 api_adapter.CLOUDRUN: _ParseAddonDisabled,
+                api_adapter.APPLICATIONMANAGER: _ParseAddonDisabled,
+                api_adapter.CLOUDBUILD: _ParseAddonDisabled,
             }),
         dest='disable_addons',
         metavar='ADDON=ENABLED|DISABLED',
@@ -88,6 +90,41 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
 {ingress}=ENABLED|DISABLED
 {dashboard}=ENABLED|DISABLED
 {istio}=ENABLED|DISABLED
+{application_manager}=ENABLED|DISABLED
+{network_policy}=ENABLED|DISABLED
+{cloudrun}=ENABLED|DISABLED
+{cloudbuild}=ENABLED|DISABLED""".format(
+    hpa=api_adapter.HPA,
+    ingress=api_adapter.INGRESS,
+    dashboard=api_adapter.DASHBOARD,
+    network_policy=api_adapter.NETWORK_POLICY,
+    istio=api_adapter.ISTIO,
+    application_manager=api_adapter.APPLICATIONMANAGER,
+    cloudrun=api_adapter.CLOUDRUN,
+    cloudbuild=api_adapter.CLOUDBUILD,
+    ))
+
+  elif release_track == base.ReleaseTrack.BETA:
+    mutex_group.add_argument(
+        '--update-addons',
+        type=arg_parsers.ArgDict(
+            spec={
+                api_adapter.INGRESS: _ParseAddonDisabled,
+                api_adapter.HPA: _ParseAddonDisabled,
+                api_adapter.DASHBOARD: _ParseAddonDisabled,
+                api_adapter.NETWORK_POLICY: _ParseAddonDisabled,
+                api_adapter.ISTIO: _ParseAddonDisabled,
+                api_adapter.CLOUDRUN: _ParseAddonDisabled,
+                api_adapter.APPLICATIONMANAGER: _ParseAddonDisabled,
+            }),
+        dest='disable_addons',
+        metavar='ADDON=ENABLED|DISABLED',
+        help="""Cluster addons to enable or disable. Options are
+{hpa}=ENABLED|DISABLED
+{ingress}=ENABLED|DISABLED
+{dashboard}=ENABLED|DISABLED
+{istio}=ENABLED|DISABLED
+{application_manager}=ENABLED|DISABLED
 {network_policy}=ENABLED|DISABLED
 {cloudrun}=ENABLED|DISABLED""".format(
     hpa=api_adapter.HPA,
@@ -95,6 +132,7 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
     dashboard=api_adapter.DASHBOARD,
     network_policy=api_adapter.NETWORK_POLICY,
     istio=api_adapter.ISTIO,
+    application_manager=api_adapter.APPLICATIONMANAGER,
     cloudrun=api_adapter.CLOUDRUN,
     ))
 
@@ -172,6 +210,17 @@ to the flag. For example:
 class Update(base.UpdateCommand):
   """Update cluster settings for an existing container cluster."""
 
+  detailed_help = {
+      'DESCRIPTION':
+          '{description}',
+      'EXAMPLES':
+          """\
+          To enable autoscaling for an existing cluster, run:
+
+            $ {command} sample-cluster --enable-autoscaling
+          """,
+  }
+
   @staticmethod
   def Args(parser):
     """Register flags for this command.
@@ -196,12 +245,20 @@ class Update(base.UpdateCommand):
     flags.AddUpdateLabelsFlag(group)
     flags.AddRemoveLabelsFlag(group)
     flags.AddNetworkPolicyFlags(group)
+    flags.AddEnableIntraNodeVisibilityFlag(group)
     group_logging_monitoring = group.add_group()
     flags.AddLoggingServiceFlag(group_logging_monitoring)
     flags.AddMonitoringServiceFlag(group_logging_monitoring)
+    flags.AddEnableBinAuthzFlag(group)
     flags.AddEnableStackdriverKubernetesFlag(group)
-    flags.AddMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddRecurringMaintenanceWindowFlags(
+        group, hidden=True, is_update=True)
     flags.AddResourceUsageExportFlags(group, is_update=True)
+    flags.AddDatabaseEncryptionFlag(group)
+    flags.AddDisableDatabaseEncryptionFlag(group)
+    flags.AddVerticalPodAutoscalingFlag(group)
+    flags.AddAutoprovisioningFlags(group)
 
   def ParseUpdateOptions(self, args, locations):
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
@@ -211,6 +268,7 @@ class Update(base.UpdateCommand):
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = \
         args.enable_resource_consumption_metering
+    opts.enable_intra_node_visibility = args.enable_intra_node_visibility
     return opts
 
   def Run(self, args):
@@ -230,6 +288,7 @@ class Update(base.UpdateCommand):
     cluster_name = args.name
     cluster_node_count = None
     cluster_zone = cluster_ref.zone
+    cluster_is_required = self.IsClusterRequired(args)
     try:
       # Attempt to get cluster for better prompts and to validate args.
       # Error is a warning but not fatal. Should only exit with a failure on
@@ -240,6 +299,8 @@ class Update(base.UpdateCommand):
       cluster_zone = cluster.zone
     except (exceptions.HttpException, apitools_exceptions.HttpForbiddenError,
             util.Error) as error:
+      if cluster_is_required:
+        raise
       log.warning(('Problem loading details of cluster to update:\n\n{}\n\n'
                    'You can still attempt updates to the cluster.\n').format(
                        console_attr.SafeText(error)))
@@ -377,8 +438,39 @@ to completion."""
         raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
     elif args.maintenance_window is not None:
       try:
-        op_ref = adapter.SetMaintenanceWindow(cluster_ref,
-                                              args.maintenance_window)
+        op_ref = adapter.SetDailyMaintenanceWindow(cluster_ref,
+                                                   cluster.maintenancePolicy,
+                                                   args.maintenance_window)
+      except apitools_exceptions.HttpError as error:
+        raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
+    elif getattr(args, 'maintenance_window_start', None) is not None:
+      try:
+        op_ref = adapter.SetRecurringMaintenanceWindow(
+            cluster_ref, cluster.maintenancePolicy,
+            args.maintenance_window_start, args.maintenance_window_end,
+            args.maintenance_window_recurrence)
+      except apitools_exceptions.HttpError as error:
+        raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
+    elif getattr(args, 'clear_maintenance_window', None):
+      try:
+        op_ref = adapter.RemoveMaintenanceWindow(cluster_ref,
+                                                 cluster.maintenancePolicy)
+      except apitools_exceptions.HttpError as error:
+        raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
+    elif getattr(args, 'add_maintenance_exclusion_end', None) is not None:
+      try:
+        op_ref = adapter.AddMaintenanceExclusion(
+            cluster_ref, cluster.maintenancePolicy,
+            args.add_maintenance_exclusion_name,
+            args.add_maintenance_exclusion_start,
+            args.add_maintenance_exclusion_end)
+      except apitools_exceptions.HttpError as error:
+        raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
+    elif getattr(args, 'remove_maintenance_exclusion', None) is not None:
+      try:
+        op_ref = adapter.RemoveMaintenanceExclusion(
+            cluster_ref, cluster.maintenancePolicy,
+            args.remove_maintenance_exclusion)
       except apitools_exceptions.HttpError as error:
         raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)
     else:
@@ -389,7 +481,7 @@ to completion."""
         options = self.ParseUpdateOptions(args, locations)
         op_ref = adapter.UpdateCluster(cluster_ref, options)
 
-    if not args.async:
+    if not args.async_:
       adapter.WaitForOperation(
           op_ref, 'Updating {0}'.format(cluster_ref.clusterId), timeout_s=1800)
 
@@ -406,6 +498,14 @@ to completion."""
         except kconfig.MissingEnvVarError as error:
           log.warning(error)
 
+  def IsClusterRequired(self, args):
+    """Returns if failure getting the cluster should be an error."""
+    return bool(
+        getattr(args, 'maintenance_window_end', False) or
+        getattr(args, 'clear_maintenance_window', False) or
+        getattr(args, 'add_maintenance_exclusion_end', False) or
+        getattr(args, 'remove_maintenance_exclusion', False))
+
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class UpdateBeta(Update):
@@ -434,7 +534,9 @@ class UpdateBeta(Update):
     flags.AddUpdateLabelsFlag(group)
     flags.AddRemoveLabelsFlag(group)
     flags.AddNetworkPolicyFlags(group)
-    flags.AddMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddRecurringMaintenanceWindowFlags(
+        group, hidden=False, is_update=True)
     flags.AddPodSecurityPolicyFlag(group)
     flags.AddEnableBinAuthzFlag(group)
     flags.AddAutoprovisioningFlags(group)
@@ -446,23 +548,12 @@ class UpdateBeta(Update):
     flags.AddWorkloadIdentityUpdateFlags(group)
     flags.AddEnableShieldedNodesFlags(group)
     flags.AddDatabaseEncryptionFlag(group)
+    flags.AddDisableDatabaseEncryptionFlag(group)
+    flags.AddReleaseChannelFlag(group, is_update=True)
 
   def ParseUpdateOptions(self, args, locations):
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
     opts.enable_pod_security_policy = args.enable_pod_security_policy
-    opts.enable_binauthz = args.enable_binauthz
-    opts.enable_autoprovisioning = args.enable_autoprovisioning
-    opts.autoprovisioning_config_file = args.autoprovisioning_config_file
-    opts.autoprovisioning_service_account = args.autoprovisioning_service_account
-    opts.autoprovisioning_scopes = args.autoprovisioning_scopes
-    opts.autoprovisioning_locations = args.autoprovisioning_locations
-    opts.min_cpu = args.min_cpu
-    opts.max_cpu = args.max_cpu
-    opts.min_memory = args.min_memory
-    opts.max_memory = args.max_memory
-    opts.min_accelerator = args.min_accelerator
-    opts.max_accelerator = args.max_accelerator
-    opts.enable_vertical_pod_autoscaling = args.enable_vertical_pod_autoscaling
     opts.istio_config = args.istio_config
     opts.resource_usage_bigquery_dataset = args.resource_usage_bigquery_dataset
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
@@ -472,7 +563,7 @@ class UpdateBeta(Update):
     opts.enable_resource_consumption_metering = args.enable_resource_consumption_metering
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
-    opts.database_encryption = flags.GetDatabaseEncryptionOption(args)
+    opts.release_channel = args.release_channel
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
@@ -512,7 +603,9 @@ class UpdateAlpha(Update):
     flags.AddNetworkPolicyFlags(group)
     flags.AddAutoprovisioningFlags(group, hidden=False)
     flags.AddAutoscalingProfilesFlag(group, hidden=True)
-    flags.AddMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
+    flags.AddRecurringMaintenanceWindowFlags(
+        group, hidden=False, is_update=True)
     flags.AddPodSecurityPolicyFlag(group)
     flags.AddEnableBinAuthzFlag(group)
     flags.AddResourceUsageExportFlags(group, is_update=True)
@@ -526,27 +619,17 @@ class UpdateAlpha(Update):
     flags.AddEnableShieldedNodesFlags(group)
     flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
     flags.AddDatabaseEncryptionFlag(group)
+    flags.AddDisableDatabaseEncryptionFlag(group)
+    flags.AddCostManagementConfigFlag(group, is_update=True)
+    flags.AddReleaseChannelFlag(group, is_update=True)
 
   def ParseUpdateOptions(self, args, locations):
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
-    opts.enable_autoprovisioning = args.enable_autoprovisioning
-    opts.autoprovisioning_config_file = args.autoprovisioning_config_file
-    opts.autoprovisioning_service_account = args.autoprovisioning_service_account
-    opts.autoprovisioning_scopes = args.autoprovisioning_scopes
-    opts.autoprovisioning_locations = args.autoprovisioning_locations
-    opts.min_cpu = args.min_cpu
-    opts.max_cpu = args.max_cpu
-    opts.min_memory = args.min_memory
-    opts.max_memory = args.max_memory
-    opts.min_accelerator = args.min_accelerator
-    opts.max_accelerator = args.max_accelerator
     opts.autoscaling_profile = args.autoscaling_profile
     opts.enable_pod_security_policy = args.enable_pod_security_policy
-    opts.enable_binauthz = args.enable_binauthz
     opts.resource_usage_bigquery_dataset = args.resource_usage_bigquery_dataset
     opts.clear_resource_usage_bigquery_dataset = \
         args.clear_resource_usage_bigquery_dataset
-    opts.enable_vertical_pod_autoscaling = args.enable_vertical_pod_autoscaling
     opts.security_profile = args.security_profile
     opts.istio_config = args.istio_config
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
@@ -555,7 +638,7 @@ class UpdateAlpha(Update):
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
     opts.enable_peering_route_sharing = args.enable_peering_route_sharing
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
-    opts.database_encryption = flags.GetDatabaseEncryptionOption(args)
+    opts.release_channel = args.release_channel
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
@@ -563,5 +646,6 @@ class UpdateAlpha(Update):
     opts.disable_workload_identity = args.disable_workload_identity
     opts.enable_shielded_nodes = args.enable_shielded_nodes
     opts.disable_default_snat = args.disable_default_snat
+    opts.enable_cost_management = args.enable_cost_management
 
     return opts
