@@ -186,7 +186,7 @@ filename:
     description: path to the certificate
     returned: changed or success
     type: str
-    sample: /tmp/certifivate-cert.pub
+    sample: /tmp/certificate-cert.pub
 info:
     description: Information about the certificate. Output of C(ssh-keygen -L -f).
     returned: change or success
@@ -365,7 +365,7 @@ class Certificate(object):
 
     def is_same_datetime(self, datetime_one, datetime_two):
 
-        # This function is for backwards compatability only because .total_seconds() is new in python2.7
+        # This function is for backwards compatibility only because .total_seconds() is new in python2.7
         def timedelta_total_seconds(timedelta):
             return ((timedelta.microseconds + 0.0 + (timedelta.seconds + timedelta.days * 24 * 3600) * 10 ** 6) / 10 ** 6)
         # try to use .total_ seconds() from python2.7
