@@ -1,6 +1,7 @@
 <?php
+
 /**
- *    Copyright (C) 2019 Frank Wall
+ *    Copyright (C) 2018 Frank Wall
  *
  *    All rights reserved.
  *
@@ -30,13 +31,26 @@ namespace OPNsense\HAProxy\Migrations;
 
 use OPNsense\Base\BaseModelMigration;
 
-class M2_7_0 extends BaseModelMigration
+class M2_6_0 extends BaseModelMigration
 {
     public function run($model)
     {
-        // Servers have an 'enabled' field now
-        foreach ($model->getNodeByReference('servers.server')->iterateItems() as $server) {
-            $server->enabled = '1';
+        // Migrate old stats user:password entries to new user management feature
+        if (!empty((string)$model->general->stats->users)) {
+            // Add new user for each entry
+            $UUIDlist = array();
+            foreach (explode(',', (string)$model->general->stats->users) as $statsuser) {
+                $olddata = explode(':', $statsuser, 2);
+                $userNode = $model->users->user->Add();
+                $userNode->name = (string)$olddata[0];
+                $userNode->description = 'stats user';
+                $userNode->password = (string)$olddata[1];
+                $userNode->enabled = 1;
+                $UUIDlist[] = $userNode->getAttributes()['uuid'];
+            }
+
+            // Add collected UUIDs to new list of allowed users
+            $model->general->stats->allowedUsers = (string)implode(',', $UUIDlist);
         }
     }
 }
